import pytest
import xml.etree.ElementTree as ET
from unittest.mock import patch
from dateutil.parser import parse

from ultrafastrss.ultrafastrss import rss_parser

def test_rss_parser():
    # Mock is_valid_url and parser.parse
    with patch('ultrafastrss.ultrafastrss.is_valid_url') as mock_is_valid_url, \
         patch('ultrafastrss.ultrafastrss.parser.parse') as mock_parse:

        # Mock return values
        mock_is_valid_url.side_effect = lambda url: url == "http://example.com"  # Valid URL check
        mock_parse.return_value = parse("2023-10-01T12:00:00")  # Mock parsed date

        # Test case 1: Valid RSS feed with all elements
        valid_rss = '''
        <rss>
            <channel>
                <item>
                    <title>  Test Title\n  </title>
                    <link>http://example.com</link>
                    <pubDate>Mon, 01 Oct 2023 12:00:00 GMT</pubDate>
                    <category>Tech</category>
                    <category>News</category>
                </item>
            </channel>
        </rss>
        '''
        result = rss_parser(valid_rss)
        expected = [{
            'title': 'Test Title',
            'link': 'http://example.com',
            'date': '2023-10-01',
            'tags': ['Tech', 'News']
        }]
        assert result == expected
        mock_is_valid_url.assert_called_with('http://example.com')
        mock_parse.assert_called_once()

      
        no_valid_url_rss = '''
        <rss>
            <channel>
                <item>
                    <title>Test No URL</title>
                    <link>invalid_url</link>
                    <pubDate>Mon, 01 Oct 2023 12:00:00 GMT</pubDate>
                </item>
            </channel>
        </rss>
        '''
        result = rss_parser(no_valid_url_rss)
        assert result == []  # Item skipped due to invalid URL
        mock_is_valid_url.assert_called_with('invalid_url')

        # Test case 4: RSS feed with missing pubDate
        no_pubdate_rss = '''
        <rss>
            <channel>
                <item>
                    <title>Test No PubDate in item</title>
                    <link>http://example.com</link>
                </item>
            </channel>
        </rss>
        '''
        result = rss_parser(no_pubdate_rss)
        expected = [{
            'title': 'Test No PubDate in item',
            'link': 'http://example.com',
            'date': '2023-10-01'
        }]
        assert result == expected

        # Test case 5: RSS feed with no categories
        no_categories_rss = '''
        <rss>
            <channel>
                <item>
                    <title>Test No Categories</title>
                    <link>http://example.com</link>
                    <pubDate>Mon, 01 Oct 2023 12:00:00 GMT</pubDate>
                </item>
            </channel>
        </rss>
        '''
        result = rss_parser(no_categories_rss)
        expected = [{
            'title': 'Test No Categories',
            'link': 'http://example.com',
            'date': '2023-10-01'
        }]
        assert result == expected

        # Test case 6: Invalid XML
        with pytest.raises(ET.ParseError):
            rss_parser('invalid xml')



import pytest
import xml.etree.ElementTree as ET
from unittest.mock import patch
from dateutil.parser import parse

from ultrafastrss.ultrafastrss import rss_parser

from importlib.resources import files

expected_result = [{'title': 'TCCing is Believing: Apple finally adds TCC events to Endpoint Security!', 
                    'date': '2025-03-27',
                    'link': 'https://objective-see.org/blog/blog_0x7F.html'},
                    {'title': 'Leaking Passwords (and more!) on macOS',
                     'date': '2025-03-20',
                     'link': 'https://objective-see.org/blog/blog_0x7E.html'}, 
                    {'title': 'The Mac Malware of 2024',
                     'date': '2025-01-01', 
                     'link': 'https://objective-see.org/blog/blog_0x7D.html'},
                    {'title': 'Restoring Reflective Code Loading on macOS', 
                     'date': '2024-12-16',
                      'link': 'https://objective-see.org/blog/blog_0x7C.html'}]


def test_rss_parser():    
    xml_file_path = files("tests.resources") / "objectivec.xml"
    with xml_file_path.open("r", encoding="utf-8") as f:
        xml_content = f.read()
        result = rss_parser(xml_content)
        assert result == expected_result


import pytest

from ultrafastrss.ultrafastrss import atom_parser
from importlib.resources import files

expected_result = [
    {
        "date": "2025-05-06",
        "link": "https://github.com/Velocidex/velociraptor/commit/99dba703bfeecce3ad1b265d175ab11e5f1eb656",
        "title": "Bugfixes: Fix build and VQL Memory limit bug (#4223)",
        "tags": []
    },
    {
        "date": "2025-05-05",
        "link": "https://github.com/Velocidex/velociraptor/commit/fe670b27ed46735d164e32a34c0220e5607e99db",
        "title": "Update Memory Acquisition artifact to store driver in user directory …",
        "tags": []
    },
    {
        "date": "2025-05-02",
        "link": "https://github.com/Velocidex/velociraptor/commit/f1c0bc6a4066ba3709ae3769f0e26e3ea8d040e8",
        "title": "Update security.md (#4219)",
        "tags": []
    },
    {
        "date": "2025-04-29",
        "link": "https://github.com/Velocidex/velociraptor/commit/5ab2cf238ce4c75c7af69e645e317270106960fb",
        "title": "Added VQL functions to manipulate secrets. (#4215)",
        "tags": []
    },
    {
        "date": "2025-04-28",
        "link": "https://github.com/Velocidex/velociraptor/commit/fe4a5e0a579d78d003eb77161ab716fd6e87cc7a",
        "title": "Giving default folder Read & Traverse permissions in Darwin installer…",
        "tags": []
    },
    {
        "date": "2025-04-28",
        "link": "https://github.com/Velocidex/velociraptor/commit/1543bef34a4e526b182f5602ea99ad00ca0639af",
        "title": "Bugfix: Allows azure authenticator to use the proxy setting. (#4214)",
        "tags": []
    },
    {
        "date": "2025-04-27",
        "title": "Delay flow error until the flow is completed. (#4210)",
        "link": "https://github.com/Velocidex/velociraptor/commit/cda0eb7672b566f04a4bb70a2c8cd0f8cedff3a1",
        "tags": []
    },
    {
        "date": "2025-04-22",
        "title": "Keep record of finished flows in client's flow manager. (#4202)",
        "link": "https://github.com/Velocidex/velociraptor/commit/a5abaee504234366a0ebe2eb114d0e02019f38d5",
        "tags": []
    },
    {
        "date": "2025-04-22",
        "title": "Avoid I/O after deleting client flows (#4179)",
        "link": "https://github.com/Velocidex/velociraptor/commit/6c27e7e2ea509c76201b64431ba71b0c3b7a2758",
        "tags": []
    },
    {
        "date": "2025-04-22",
        "title": "Fixed tests",
        "link": "https://github.com/Velocidex/velociraptor/commit/22f8245dfe0b93d818acdbbb2e66791ffc2c06e5",
        "tags": []
    },
    {
        "date": "2025-04-22",
        "title": "Bugfix: Allow API client to push events without high permissions.",
        "link": "https://github.com/Velocidex/velociraptor/commit/3f050cdae7d0e61ccf7e631daa2cb001a2db9c2d",
        "tags": []
    },
    {
        "date": "2025-04-19",
        "title": "Bugfix: Move atomic fields to the start of the struct (#4197)",
        "link": "https://github.com/Velocidex/velociraptor/commit/bd0017a76271a8ede39f2661af26de9efb9d3c58",
        "tags": []
    },
    {
        "date": "2025-04-19",
        "title": "Bugfix: Implement better elastic index sanitization (#4196)",
        "link": "https://github.com/Velocidex/velociraptor/commit/a53a69328d2f8ca83eabac38009d8507c50be3bf",
        "tags": []
    },
    {
        "date": "2025-04-18",
        "title": "Bugfix: Do not complete collection on error.  (#4194)",
        "link": "https://github.com/Velocidex/velociraptor/commit/3de202739fa04af1e519298a08fa464b3a733e74",
        "tags": []
    },
    {
        "date": "2025-04-18",
        "title": "Bump http-proxy-middleware from 2.0.7 to 2.0.9 in /gui/velociraptor (…",
        "link": "https://github.com/Velocidex/velociraptor/commit/f082efe463a052b5bb663c9c8f13f106511ad316",
        "tags": []
    },
    {
        "date": "2025-04-18",
        "title": "Bugfix: Close tempfile before removing (#4191)",
        "link": "https://github.com/Velocidex/velociraptor/commit/1dbc9e0de8e7c9d546933d1e89a2faf752af8aef",
        "tags": []
    },
    {
        "date": "2025-04-17",
        "title": "Bugfix: Fixed bug in Server.Orgs.NewOrg",
        "link": "https://github.com/Velocidex/velociraptor/commit/62bb7709c55d7dd316231643ed56ed480cf1b87e",
        "tags": []
    },
    {
        "date": "2025-04-17",
        "title": "fix syntax error in pattern of TempFile",
        "link": "https://github.com/Velocidex/velociraptor/commit/9097291080c9777c2748c86a6b9e1a733c7b74fc",
        "tags": []
    },
    {
        "date": "2025-04-16",
        "title": "Set the user avatar in the datastore (#4180)",
        "link": "https://github.com/Velocidex/velociraptor/commit/f9bf853b591e8de845836ffbddca6c4e70a55628",
        "tags": []
    },
    {
        "date": "2025-04-15",
        "title": "Bump github.com/gorilla/csrf from 1.6.2 to 1.7.3 (#4176)",
        "link": "https://github.com/Velocidex/velociraptor/commit/cffeae708d886d973d24ab3efccff71b48b6cdf5",
        "tags": []
    }
]



def test_github_atom_parser():    
    xml_file_path = files("tests.resources") / "github.atom"
    with xml_file_path.open("r", encoding="utf-8") as f:
        xml_content = f.read()
        result = atom_parser(xml_content)
        print(result)
        assert result == expected_result


import pytest
import xml.etree.ElementTree as ET
from unittest.mock import patch
from dateutil.parser import parse

from ultrafastrss.ultrafastrss import rss_parser

from importlib.resources import files

expected_result = [
    {
        "title": "xkcd 1970: Name Dominoes",
        "link": "https://github.com/norvig/pytudes/blob/master/ipynb/xkcd-Name-Dominoes.ipynb",        
        "date": "2018-03-21"
    },
    {
        "title": "Functional Lifestyles Training",
        "link": "https://medium.com/@peternorvig/functional-lifestyles-training-47984a3cd2ba",       
        "date": "2018-04-01"
    },   
    {
        "title": "JScheme",
        "link": "http://norvig.com/jscheme.html",       
        "date": "2005-08-08"
    }
]


def test_rss_parser():    
    xml_file_path = files("tests.resources") / "norvig_rssfeed.xml"
    with xml_file_path.open("r", encoding="utf-8") as f:
        xml_content = f.read()
        result = rss_parser(xml_content)
        assert result == expected_result


import pytest

from ultrafastrss.ultrafastrss import atom_parser
from importlib.resources import files

expected_result = [
    {
        "title": "Radical Open Innovation News week 47-2024",
        "date": "2024-11-22",
        "link": "https://www.bm-support.org/roi-news-week27-2024/",
        "tags": ["Blogs", "Innovation News"]
    },
    {
        "title": "Radical Open Innovation News week 1-2024",
        "date": "2024-01-05",
        "link": "https://www.bm-support.org/radical-open-innovation-news-week-1-2024/",
        "tags": ["Blogs", "Innovation News"]
    },
    {
        "title": "Radical Open Innovation News week 26-2023",
        "date": "2023-06-30",
        "link": "https://www.bm-support.org/roi-news-week26-2023/",
        "tags": ["Blogs", "Innovation News"]
    },
    {
        "title": "Radical Open Innovation News week 41-2022",
        "date": "2022-10-09",
        "link": "https://www.bm-support.org/roi-news-week41-2021/",
        "tags": ["Blogs", "Innovation News"]
    },
    {
        "title": "Radical Open Innovation News week 16-2021",
        "date": "2021-04-25",
        "link": "https://www.bm-support.org/roi-news-week16-2021/",
        "tags": ["Blogs", "Innovation News"]
    },
    {
        "title": "Radical Open Innovation News week 14-2021",
        "date": "2021-04-10",
        "link": "https://www.bm-support.org/roi-news-week14-2021/",
        "tags": ["Blogs", "Innovation News"]
    },
    {
        "title": "Radical Open Innovation News week 12-2021",
        "date": "2021-03-27",
        "link": "https://www.bm-support.org/roi-news-week12-2021/",
        "tags": ["Blogs", "Innovation News"]
    },
    {
        "title": "Radical Open Innovation News week 10-2021",
        "date": "2021-03-12",
        "link": "https://www.bm-support.org/roi-news-week10-2021/",
        "tags": ["Blogs", "Innovation News"]
    },
    {
        "title": "Radical Open Innovation News week 8-2021",
        "date": "2021-02-26",
        "link": "https://www.bm-support.org/roi-news-week8-2021/",
        "tags": ["Blogs", "Innovation News"]
    },
    {
        "title": "Radical Open Innovation News week 6-2021",
        "date": "2021-02-15",
        "link": "https://www.bm-support.org/roi-news-week6-2021/",
        "tags": ["Blogs", "Innovation News"]
    }
]


def test_rss_parser():    
    xml_file_path = files("tests.resources") / "roi.atom"
    with xml_file_path.open("r", encoding="utf-8") as f:
        xml_content = f.read()
        result = atom_parser(xml_content)
        assert result == expected_result


import pytest
import xml.etree.ElementTree as ET
from ultrafastrss.ultrafastrss import rdf_parser, parse_date, is_valid_url


def test_rdf_parser():
    """
    Tests the rdf_parser function with various XML inputs.
    """

    # Test case 1: Valid XML with multiple items
    valid_xml_multiple_items = """
    <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
             xmlns="http://purl.org/rss/1.0/"
             xmlns:dc="http://purl.org/dc/elements/1.1/">
      <channel rdf:about="http://example.com/channel">
        <items>
          <rdf:Seq>
            <rdf:li rdf:resource="http://example.com/item1"/>
            <rdf:li rdf:resource="http://example.com/item2"/>
          </rdf:Seq>
        </items>
      </channel>
      <item rdf:about="http://example.com/item1">
        <title>  First Item Title  </title>
        <link>http://example.com/first-item</link>
        <dc:date>2023-01-15T12:30:00Z</dc:date>
      </item>
      <item rdf:about="http://example.com/item2">
        <title>Second Item Title</title>
        <link>https://example.com/second-item</link>
        <dc:date>2023-02-20T09:00:00Z</dc:date>
      </item>
    </rdf:RDF>
    """
    expected_multiple_items = [
        {"title": "First Item Title", "link": "http://example.com/first-item", "date": "2023-01-15", "tags": []},
        {"title": "Second Item Title", "link": "https://example.com/second-item", "date": "2023-02-20", "tags": []},
    ]
    assert rdf_parser(valid_xml_multiple_items) == expected_multiple_items

    # Test case 2: Valid XML with no items
    valid_xml_no_items = """
    <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
             xmlns="http://purl.org/rss/1.0/"
             xmlns:dc="http://purl.org/dc/elements/1.1/">
      <channel rdf:about="http://example.com/channel">
        <items>
          <rdf:Seq/>
        </items>
      </channel>
    </rdf:RDF>
    """
    assert rdf_parser(valid_xml_no_items) == []

    # Test case 3: Invalid XML data
    invalid_xml = "This is not XML data."
    assert rdf_parser(invalid_xml) == []

    

    # Test case 5: XML with invalid link
    xml_invalid_link = """
    <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
             xmlns="http://purl.org/rss/1.0/"
             xmlns:dc="http://purl.org/dc/elements/1.1/">
      <item rdf:about="http://example.com/item6">
        <title>Item with Invalid Link</title>
        <link>invalid-url</link>
        <dc:date>2023-04-05T11:00:00Z</dc:date>
      </item>
      <item rdf:about="http://example.com/item7">
        <title>Item with Valid Link</title>
        <link>http://example.com/valid-link</link>
        <dc:date>2023-04-05T11:00:00Z</dc:date>
      </item>
    </rdf:RDF>
    """
    expected_invalid_link = [
        {"title": "Item with Valid Link", "link": "http://example.com/valid-link", "date": "2023-04-05", "tags": []},
    ]
    # The item with "invalid-url" should be filtered out because data["link"] becomes "None"
    assert rdf_parser(xml_invalid_link) == expected_invalid_link

    # Test case 6: XML with empty text for elements
    xml_empty_text = """
    <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
             xmlns="http://purl.org/rss/1.0/"
             xmlns:dc="http://purl.org/dc/elements/1.1/">
      <item rdf:about="http://example.com/item8">
        <title></title>
        <link></link>
        <dc:date></dc:date>
      </item>
      <item rdf:about="http://example.com/item9">
        <title>Item 9 Title</title>
        <link>http://example.com/item9-link</link>
        <dc:date>2023-05-10T08:00:00Z</dc:date>
      </item>
    </rdf:RDF>
    """
    expected_empty_text = [
        {"title": "None", "link": "None", "date": "None", "tags": []}, # This item will be filtered out due to link being "None"
        {"title": "Item 9 Title", "link": "http://example.com/item9-link", "date": "2023-05-10", "tags": []},
    ]
    # The first item will be filtered out because its link becomes "None"
    assert rdf_parser(xml_empty_text) == [
        {"title": "Item 9 Title", "link": "http://example.com/item9-link", "date": "2023-05-10", "tags": []},
    ]

    # Test case 7: XML with only whitespace in title
    xml_whitespace_title = """
    <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
             xmlns="http://purl.org/rss/1.0/"
             xmlns:dc="http://purl.org/dc/elements/1.1/">
      <item rdf:about="http://example.com/item10">
        <title>   </title>
        <link>http://example.com/item10-link</link>
        <dc:date>2023-06-15T14:00:00Z</dc:date>
      </item>
    </rdf:RDF>
    """
    expected_whitespace_title = [
        {"title": "", "link": "http://example.com/item10-link", "date": "2023-06-15", "tags": []},
    ]
    assert rdf_parser(xml_whitespace_title) == expected_whitespace_title





import pytest
import xml.etree.ElementTree as ET
from unittest.mock import patch
from dateutil.parser import parse

from ultrafastrss.ultrafastrss import rss_parser

from importlib.resources import files

expected_result = [{'title': 'Shift left: \xa0from complex and reactive to simple and proactive', 'link': 'https://nocomplexity.com/shift-left/', 'date': '2025-04-18', 'tags': ['Security', 'architecture']}, {'title': 'Open ML/AI News week 13-2025', 'link': 'https://nocomplexity.com/open-ai-news-week-13-2025/', 'date': '2025-03-29', 'tags': ['Machine Learning News', 'machine learning']}, {'title': 'Open Security News week 12-2025', 'link': 'https://nocomplexity.com/security-news-week-12-2025/', 'date': '2025-03-23', 'tags': ['Security News', 'Security']}, {'title': 'Open Security News week 10-2025', 'link': 'https://nocomplexity.com/security-news-week-10-2025/', 'date': '2025-03-02', 'tags': ['Security', 'Security News']}, {'title': 'Open Security News week 8-2025', 'link': 'https://nocomplexity.com/security-news-week-8-2025/', 'date': '2025-02-18', 'tags': ['Security News', 'Security']}, {'title': 'Open ML/AI News week 7-2025', 'link': 'https://nocomplexity.com/open-ml-news-week-7-2025/', 'date': '2025-02-15', 'tags': ['Machine Learning News', 'machine learning']}, {'title': 'Simplify Python', 'link': 'https://nocomplexity.com/simplifypython/', 'date': '2025-01-21', 'tags': ['Information Technology', 'Python']}, {'title': 'Pushing complexity to users', 'link': 'https://nocomplexity.com/pushing-complexity-to-users/', 'date': '2025-01-17', 'tags': ['Business', 'Information Technology', 'architecture', 'Complexity', 'Simple']}, {'title': 'Simplify IT', 'link': 'https://nocomplexity.com/simplifyit/', 'date': '2025-01-17', 'tags': ['Business', 'Information Technology', 'architecture', 'Simple', 'Software']}, {'title': 'Finding duplicates in a collection: A solved problem!', 'link': 'https://nocomplexity.com/finding-duplicates/', 'date': '2025-01-10', 'tags': ['Information Technology', 'architecture', 'Simple']}]


def test_rss_parser():    
    xml_file_path = files("tests.resources") / "nocxrss.xml"
    with xml_file_path.open("r", encoding="utf-8") as f:
        xml_content = f.read()
        result = rss_parser(xml_content)
        assert result == expected_result


"""
Function to test only parse_feed

Generated test - so beware!

Explanation:

    Mocking: The rss_parser, atom_parser, and rdf_parser functions are mocked to avoid testing their implementation. They return predefined values to test the logic of parse_feed.
    Test Cases:
        RSS 2.0: Tests if the function correctly identifies an RSS feed and calls rss_parser.
        Atom: Tests if the function identifies an Atom feed and calls atom_parser.
        RDF: Tests if the function identifies an RDF feed and calls rdf_parser.
        Unknown Feed: Tests if the function returns 'Unknown feed type' for an unrecognized root tag.
        Invalid XML: Tests if the function raises an ET.ParseError for malformed XML.
    Assertions: Verifies that the correct parser is called and the expected output is returned.
    Module Reference: Replace 'your_module' with the actual module name where parse_feed and the parser functions are defined.

This test ensures the parse_feed function correctly handles different feed types and errors. Save it in a file like test_parse_feed.py and run with pytest.
"""


import pytest
import xml.etree.ElementTree as ET
from unittest.mock import patch
from ultrafastrss.ultrafastrss import parse_feed

def test_parse_feed():
    # Mock the parser functions
    with patch('ultrafastrss.ultrafastrss.rss_parser') as mock_rss, \
         patch('ultrafastrss.ultrafastrss.atom_parser') as mock_atom, \
         patch('ultrafastrss.ultrafastrss.rdf_parser') as mock_rdf:

        # Mock return values
        mock_rss.return_value = [{"title": "RSS feed"}]
        mock_atom.return_value = [{"title": "Atom feed"}]
        mock_rdf.return_value = [{"title": "RDF feed"}]

        # Test case 1: RSS 2.0 feed
        rss_xml = '<rss><channel><item><title>RSS feed</title></item></channel></rss>'
        result = parse_feed(rss_xml)
        mock_rss.assert_called_once_with(rss_xml)
        assert result == [{"title": "RSS feed"}]

        # Test case 2: Atom feed
        atom_xml = '<feed xmlns="http://www.w3.org/2005/Atom"><entry><title>Atom feed</title></entry></feed>'
        result = parse_feed(atom_xml)
        mock_atom.assert_called_once_with(atom_xml)
        assert result == [{"title": "Atom feed"}]

        # Test case 3: RDF feed
        rdf_xml = '<RDF xmlns="http://www.w3.org/1999/02/22-rdf-syntax-ns#"><item><title>RDF feed</title></item></RDF>'
        result = parse_feed(rdf_xml)
        mock_rdf.assert_called_once_with(rdf_xml)
        assert result == [{"title": "RDF feed"}]

        # Test case 4: Unknown feed type
        unknown_xml = '<invalid>Invalid feed</invalid>'
        result = parse_feed(unknown_xml)
        assert result == 'Unknown feed type'

        # Test case 5: Invalid XML
        with pytest.raises(ET.ParseError):
            parse_feed('invalid xml')




"""Sample to test that date is retrieved from <lastBuildDate> instead of None, when items have no date field"""

import pytest
from unittest.mock import patch

from ultrafastrss.ultrafastrss import rss_parser

from importlib.resources import files

expected_result = [{'title': 'RFC 9728: OAuth 2.0 Protected Resource Metadata',
  'link': 'https://www.rfc-editor.org/info/rfc9728',
  'date': '2025-04-23'},
 {'title': "RFC 9766: Extensions for Weak Cache Consistency in NFSv4.2's Flexible File Layout",
  'link': 'https://www.rfc-editor.org/info/rfc9766',
  'date': '2025-04-23'},
 {'title': 'RFC 9750: The Messaging Layer Security (MLS) Architecture',
  'link': 'https://www.rfc-editor.org/info/rfc9750',
  'date': '2025-04-23'},
 {'title': 'RFC 9765: RADIUS/1.1: Leveraging Application-Layer Protocol Negotiation (ALPN) to Remove MD5',
  'link': 'https://www.rfc-editor.org/info/rfc9765',
  'date': '2025-04-23'},
 {'title': 'RFC 9767: Grant Negotiation and Authorization Protocol Resource Server Connections',
  'link': 'https://www.rfc-editor.org/info/rfc9767',
  'date': '2025-04-23'},
 {'title': 'RFC 9753: Extension for Stateful PCE to Allow Optional Processing of Path Computation Element Communication Protocol (PCEP) Objects',
  'link': 'https://www.rfc-editor.org/info/rfc9753',
  'date': '2025-04-23'},
 {'title': 'RFC 9761: Manufacturer Usage Description (MUD) for TLS and DTLS Profiles for Internet of Things (IoT) Devices',
  'link': 'https://www.rfc-editor.org/info/rfc9761',
  'date': '2025-04-23'},
 {'title': 'RFC 9752: Conveying Vendor-Specific Information in the Path Computation Element Communication Protocol (PCEP) Extensions for Stateful PCE',
  'link': 'https://www.rfc-editor.org/info/rfc9752',
  'date': '2025-04-23'},
 {'title': 'RFC 9719: YANG Data Model for Routing in Fat Trees (RIFT)',
  'link': 'https://www.rfc-editor.org/info/rfc9719',
  'date': '2025-04-23'},
 {'title': 'RFC 9764: Bidirectional Forwarding Detection (BFD) Encapsulated in Large Packets',
  'link': 'https://www.rfc-editor.org/info/rfc9764',
  'date': '2025-04-23'},
 {'title': 'RFC 9692: RIFT: Routing in Fat Trees',
  'link': 'https://www.rfc-editor.org/info/rfc9692',
  'date': '2025-04-23'},
 {'title': 'RFC 9696: Routing in Fat Trees (RIFT) Applicability and Operational Considerations',
  'link': 'https://www.rfc-editor.org/info/rfc9696',
  'date': '2025-04-23'},
 {'title': 'RFC 9759: Unified Time Scaling for Temporal Coordination Frameworks',
  'link': 'https://www.rfc-editor.org/info/rfc9759',
  'date': '2025-04-23'},
 {'title': 'RFC 9726: Operational Considerations for Use of DNS in Internet of Things (IoT) Devices',
  'link': 'https://www.rfc-editor.org/info/rfc9726',
  'date': '2025-04-23'},
 {'title': 'RFC 9777: Multicast Listener Discovery Version 2 (MLDv2) for IPv6',
  'link': 'https://www.rfc-editor.org/info/rfc9777',
  'date': '2025-04-23'}]

def test_rss_parser():    
    xml_file_path = files("tests.resources") / "rfcrss.xml"
    with xml_file_path.open("r", encoding="utf-8") as f:
        xml_content = f.read()
        result = rss_parser(xml_content)
        assert result == expected_result


import pytest
import asyncio
from ultrafastrss.ultrafastrss import process_rssfeeds

import json


def test_integration(tmp_path):
    """Test0 - the async URL check with my RSS URLs
    tmp_path is a pytest fixture that provides a temporary directory unique to the test, managed by pytest.
    """
    urls = [
        'https://nocomplexity.com/rss',
        'https://www.eff.org/rss/updates.xml',
        'https://www.freebsd.org/news/rss.xml',
        'https://ubuntu.com/blog/feed',
        'https://nlnet.nl/feed.atom',
        'https://j3s.sh/feed.atom',
        'https://blog.research.google/atom.xml'
    ]
    
    # Run the async RSS feed processing
    results = asyncio.run(process_rssfeeds(urls))
    
    # Assert that results is a list
    assert isinstance(results, list), "Results should be a list"
    
    # Save results rss_results.json
    d = tmp_path / "testoutput"
    d.mkdir()
    output_path = d / "rss_results.json"
    try:
        with output_path.open('w', encoding='utf-8') as f:
            json.dump(results, f, indent=2, ensure_ascii=False)
            print(f"Results saved to: file:/{output_path}")  # Print the message
    except IOError as e:
        pytest.fail(f"Failed to write results to {output_path}: {e}")
    
    # Optionally, verify the file was written
    assert output_path.exists(), f"JSON file was not created at: {output_path}"



"""
License GPL3

(C) 2024-2025 Created by Maikel Mardjan - https://nocomplexity.com/

Date created    : 11-12-2024
Last updated    : See git history

Simple FAST Async RSS parser
"""

import asyncio
import gzip
import zlib
import brotli

from urllib.request import Request, urlopen
from urllib.error import URLError, HTTPError

# from datetime import datetime
from dateutil import parser
from dateutil.tz import tzoffset
import xml.etree.ElementTree as ET
import re

import datetime


nocxheaders = {
    "user-agent": "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:143.0) Gecko/20100101 Firefox/143.0",
    "Accept": "text/html, application/xhtml+xml, application/xml;q=0.9, */*;q=0.8",
    "Accept-Encoding": "gzip, deflate,br",
    "Connection": "keep-alive",  
    "Upgrade-Insecure-Requests": "1",  
}
nocxtimeout = 7

# Define known timezone mappings
tzinfos = {
    "GMT": tzoffset("UTC", 0),
    "UT": tzoffset("UTC", 0),
    "UTC": tzoffset("UTC", 0),
    "PST": tzoffset("PST", -8 * 3600),  # Pacific Standard Time: UTC-8
    "PDT": tzoffset("PDT", -7 * 3600),  # Pacific Daylight Time: UTC-7
    "EST": tzoffset("EST", -5 * 3600),  # Eastern Standard Time: UTC-5
    "EDT": tzoffset("EDT", -4 * 3600),  # Eastern Daylight Time: UTC-4
    "CST": tzoffset("CST", -6 * 3600),  # Central Standard Time: UTC-6
    "CDT": tzoffset("CDT", -5 * 3600),  # Central Daylight Time: UTC-5
    "MST": tzoffset("MST", -7 * 3600),  # Mountain Standard Time: UTC-7
    "MDT": tzoffset("MDT", -6 * 3600),  # Mountain Daylight Time: UTC-6
    "HST": tzoffset("HST", -10 * 3600),  # Hawaii Standard Time: UTC-10
}


async def async_feedparser(url):
    """
    async version to parse RSS feeds in a 2025 way.
    Checks the status of a given URL and reports HTTP status codes or DNS errors.

    Args:
        url (str): The URL to check.

    Returns:
        tuple: A tuple containing the URL and either the status code or an error message.
    """

    def feedparser():
        try:
            request = Request(url, headers=nocxheaders)
            with urlopen(request, timeout=nocxtimeout) as response:
                content: bytes = response.read()
                content_encoding = response.headers.get("Content-Encoding")
                # Decompress content based on encoding
                if content_encoding == "gzip":
                    content = gzip.decompress(content)
                elif content_encoding == "deflate":
                    content = zlib.decompress(content, -zlib.MAX_WBITS)
                elif content_encoding == "br":  # Handle Brotli encoding
                    content = brotli.decompress(content)
                elif content_encoding not in [None]:
                    raise ValueError(f"Unexpected content encoding: {content_encoding}")
                # Decode content based on charset
                content_type = response.headers.get("Content-Type", "")
                content_charset = response.headers.get_content_charset()
                # If no charset is provided, extract it from Content-Type header
                if not content_charset and "charset=" in content_type:
                    content_charset = content_type.split("charset=")[-1].split(";")[0]
                # Default to UTF-8 if no charset is found
                content_charset = content_charset or "utf-8"
                # Decode with fallback for invalid bytes
                xml_content = content.decode(content_charset, errors="replace")
                parse_result = parse_feed(xml_content)
            return parse_result
        except HTTPError as e:
            print(f"{url} : HTTP Error: {e.code} {e.reason}")
        except URLError as e:
            print(f"{url} : URL Error: {e.reason}")
        except Exception as e:
            print(f"{url} : Unexpected Error: {str(e)}")

    return await asyncio.to_thread(feedparser)


def parse_feed(xml_content):
    """Feed parser
    Parses RSS feeds (atom 2.0) and returns JSON

    Args:
        xml_content: xml of RSS

    Returns:
        json: rss content as json
    """
    root = ET.fromstring(xml_content)
    # Check for RSS 2.0
    if root.tag == "rss":
        # Use the RSS parser
        filedata = rss_parser(xml_content)
    elif root.tag == "{http://www.w3.org/2005/Atom}feed":
        filedata = atom_parser(xml_content)
    elif root.tag == "{http://www.w3.org/1999/02/22-rdf-syntax-ns#}RDF":
        filedata = rdf_parser(xml_content)
    else:
        # Unknown feed type - or I do not want this complexity
        filedata = "Unknown feed type"
    return filedata


def rss_parser(xml_data):
    """Parse the RSS XML"""
    ns = {"dc": "http://purl.org/dc/elements/1.1/"}  # namespace for dc:date
    root = ET.fromstring(xml_data)
    items = root.findall(".//item")
    filedata = []
    for entry in items:
        data = {}
        title_element = entry.find("title")
        if title_element is not None:
            title = title_element.text
            # Remove all newlines and leading/trailing spaces
            cleaned_title = str(title).replace("\n", "").strip()
            data["title"] = cleaned_title
        else:
            # malformed RSS! So no title but it is a item.
            data["title"] = "None"
        link_element = entry.find("link")
        if link_element is not None:
            link = link_element.text
            # Check if link is valid
            if is_valid_url(link):
                data["link"] = link
            else:
                data["link"] = "Invalid URL"
        else:
            link_element = entry.find("guid")
            link = link_element.text
            data["link"] = link
        pub_date_element = entry.find("pubDate")
        dc_date_element = entry.find("dc:date", ns)
        date_text = None
        # Check if the 'pubDate' element exists and has text
        if pub_date_element is not None and pub_date_element.text:
            date_text = pub_date_element.text.strip()
        elif dc_date_element is not None and dc_date_element.text:
            date_text = dc_date_element.text.strip()
        else:
            # Handle case when <pubDate> or <dc_date> is not found
            fallback_date = get_last_build_date(xml_data)
            if fallback_date:
                date_text = fallback_date.strip()

        if date_text:
            try:
                date_obj = parser.parse(date_text, tzinfos=tzinfos)
                formatted_date = date_obj.strftime("%Y-%m-%d")
                data["date"] = formatted_date
            except Exception:
                # set current date instead of Invalid [open issue]
                today_date = (datetime.datetime.now()).date()
                data["date"] = str(today_date)
                # data['date'] = 'Invalid date'
        else:
            data["date"] = "None"
        # Now check for categories in feed
        categories = entry.findall("category")
        if categories:
            tags = []
            for category in categories:
                tags.append(category.text)
                data["tags"] = tags
        # Only store items with a valid URL
        if is_valid_url(link):
            filedata.append(data)
    return filedata


def get_last_build_date(xml_content):
    """
    Retrieves the <lastBuildDate> from an RSS XML string.

    Args:
        xml_content (str): The RSS XML content as a string.

    Returns:
        str: The text of the <lastBuildDate> element, or 'None' if not found or if parsing fails.

    Raises:
        ET.ParseError: If the XML is malformed and cannot be parsed.
    """
    try:
        # Parse the XML content
        root = ET.fromstring(xml_content)

        # Find the <lastBuildDate> element in the <channel>
        last_build_date_element = root.find(".//channel/lastBuildDate")

        # Check if the element exists and has text
        if last_build_date_element is not None and last_build_date_element.text:
            return last_build_date_element.text.strip()
        else:
            return "None"

    except ET.ParseError:
        return "None"


def atom_parser(xml_data):
    """Parse the Atom XML"""
    root = ET.fromstring(xml_data)
    # Namespace handling
    namespace = {
        "atom": "http://www.w3.org/2005/Atom",
        "xhtml": "http://www.w3.org/1999/xhtml",
        "media": "http://search.yahoo.com/mrss/",
    }  # Added media namespace
    # Find all entries using namespace-aware XPath
    items = root.findall(".//atom:entry", namespaces=namespace)
    # Extract data
    parsed_entries = []
    for item in items:
        data = {}
        # Find the <title> element
        title_element = item.find("atom:title", namespaces=namespace)
        if title_element is not None:
            if title_element.get("type") == "xhtml":
                # Find the nested <div> element
                div_element = title_element.find("xhtml:div", namespaces=namespace)
                if div_element is not None:
                    title = div_element.text
                else:
                    title = None  # Handle case where div is missing
            else:
                title = title_element.text
            cleaned_title = str(title).replace("\n", "").strip()
            data["title"] = cleaned_title
        else:
            data["title"] = None  # Handle case where title is missing

        # Now link: Prioritize atom:link with rel="alternate"
        link_element = item.find("atom:link[@rel='alternate']", namespaces=namespace)
        if link_element is not None:
            link = link_element.get("href")
        elif item.find("atom:id", namespaces=namespace) is not None:
            # Fallback to atom:id if no alternate link is found (though less common for primary link)
            link = item.find("atom:id", namespaces=namespace).text
        else:
            link = None
        data["link"] = link

        # published element
        published = item.find("atom:published", namespaces=namespace)
        if published is not None:
            simplified_date = parse_date(published)
        else:
            published = item.find("atom:updated", namespaces=namespace)
            if published is not None:
                simplified_date = parse_date(published)
            else:
                simplified_date = "None"
        data["date"] = simplified_date

        categories = item.findall("atom:category", namespaces=namespace)
        if categories:
            tags = [
                category.get("term") for category in categories if category.get("term")
            ]
            data["tags"] = tags
        else:
            data["tags"] = []  # Set tags to empty list if no categories found

        # Only store items with a valid URL
        if link is not None and is_valid_url(
            link
        ):  # Ensure link is not None before checking validity
            parsed_entries.append(data)
        # Return parsed entries
    return parsed_entries


def rdf_parser(xml_data):
    """Parse RDF XML."""
    try:
        root = ET.fromstring(xml_data)
    except ET.ParseError:
        return []
    namespaces = {
        "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
        "dc": "http://purl.org/dc/elements/1.1/",
        "": "http://purl.org/rss/1.0/",
    }
    filedata = []
    for item in root.findall(".//item", namespaces):
        data = {}
        title = item.find("title", namespaces)
        link = item.find("link", namespaces)
        date_field = item.find("dc:date", namespaces)
        data["title"] = (
            title.text.replace("\n", "").strip()
            if title is not None and title.text
            else "None"
        )
        data["link"] = (
            link.text if link is not None and is_valid_url(link.text) else "None"
        )
        data["date"] = (
            parse_date(date_field)
            if date_field is not None and date_field.text
            else "None"
        )
        data["tags"] = []  # Add for consistency
        if data["link"] != "None":
            filedata.append(data)
    return filedata


def parse_date(date_field):
    """Make date field simple - or do not use this if you want need exact timezones with time
    Input: XML date field

    Output: None or date in format string "Y-m-d"
    """
    # Check if the 'pubDate' element exists and has text
    if date_field is not None and date_field.text:
        published = date_field.text
        # Input: Format date string, can be different for RSS feeds and Remove 'GMT'
        standardized_published = published.strip()  # Ensure no leading/trailing spaces
        # Parse date with tzinfos for known timezones
        date_obj = parser.parse(standardized_published, tzinfos=tzinfos)
        formatted_date = date_obj.strftime("%Y-%m-%d")  # Format as YYYY-MM-DD
        simplified_date = str(formatted_date)
        return simplified_date
    else:
        # Handle case when pubDate is not found
        return "None"


async def process_rssfeeds(urls):
    """Process multiple RSS Feeds URLs asynchronously."""
    tasks = [async_feedparser(url) for url in urls]
    results = await asyncio.gather(*tasks)
    return results


def is_valid_url(url):
    """
    Check if a string is a valid URL starting with http:// or https://.

    Args:
        url (str): The string to check.

    Returns:
        bool: True if the string is a valid URL, False otherwise.
    """
    if not isinstance(url, str) or not url.strip():
        # Reject non-string inputs or empty/whitespace-only strings
        return False

    url_regex = re.compile(
        r"^(https?://)"  # Must start with http:// or https://
        r"(([a-zA-Z0-9-]+\.)+[a-zA-Z]{2,})"  # Domain with valid TLD
        r"(:\d+)?"  # Optional port
        r"(/[a-zA-Z0-9\-._~:/?#[\]@!$&\'()*+,;=]*)?$"  # Optional path, query, and fragment
    )
    return bool(url_regex.match(url))
